' This program is provided as Freeware for personal, educational
' and non-commercial use under a GNU General Public License.
'
' Written by Mike Perks (basicx@austin.rr.com), 2005
'
' Robot Electronics TPA81 Infrared Thermal Sensor
' (see http://www.robot-electronics.co.uk/htm/tpa81tech.htm)
'
' Written for ZBasic Application Note AN-212
'
Option Explicit
Option Strict On

' *************************************************************
' Define Public Constant Values for TPA81
'
' *************************************************************
Public Const TPA81_UNITS_CENTIGRADE as Byte 		= 80
Public Const TPA81_UNITS_FAHRENHEIT as Byte 		= 81

' *************************************************************
' Sub InitTPA81
' 
' Initialize the TPA81
' Parm1 is the I2C address of the TPA81
' Parm2 is the temperature units (Centigrade or Fahrenheit)
'**************************************************************
Public Sub InitTPA81(ByVal addr as Byte, ByVal units as Byte)
	' save data for later use
	address = addr
	temperatureUnits = units

	' open the I2C channel
	Call OpenI2C(channel, sdaPin, sclPin)
End Sub	

' *************************************************************
' Sub TermTPA81
' 
' Terminate using the TPA81 by closing the I2C channel
'**************************************************************
Public Sub TermTPA81()
	Call CloseI2C(channel)
End Sub

' *************************************************************
' Function GetTPA81AmbientTemperature
' 
' Returns the current ambient temperature in the units specified
' by InitTPA81
'**************************************************************
Public Function GetTPA81AmbientTemperature() as Byte
	Dim rc as Integer
	Dim reg as Byte
	
	' get ambient temperature in centigrade
	reg = REG_AMBIENT
	rc = I2CCmd(channel, address, CByte(SizeOf(reg)), reg, 1, GetTPA81AmbientTemperature)
	If rc < 1 Then
		Debug.Print "GetTPA81AmbientTemperature i2cmd returned ";CStr(rc)
		GetTPA81AmbientTemperature = 0
		Exit Function
	End If	
	
	' convert to fahrenheit if necessary
	If temperatureUnits = TPA81_UNITS_FAHRENHEIT Then
		GetTPA81AmbientTemperature = CByte(CSng(GetTPA81AmbientTemperature) * 1.8 + 32.0)
	End If
End Function


' *************************************************************
' Function GetTPA81Temperatures
' 
' Gets the array of temperatures in the units specified by InitTPA81
' 
' Parm1 ByReference array to hold temperatures
'**************************************************************
Public Sub GetTPA81Temperatures(ByRef temps() as Byte)
	Dim rc as Integer
	Dim reg as Byte
	
	' get the temperature array
	reg = REG_TEMP
	rc = I2CCmd(channel, address, CByte(SizeOf(reg)), reg, 8, temps)
	If rc < 1 Then
		Debug.Print "GetTPA81Temperatures i2cmd returned ";CStr(rc)
	End If	
	
	' convert the array of temperatures to fahrenheit if necessary
	If temperatureUnits = TPA81_UNITS_FAHRENHEIT Then
		Dim i as Byte
		For i=1 to 8
			temps(i) = CByte(CSng(temps(i)) * 1.8 + 32.0)
		Next
	End If
End Sub


' *************************************************************
' Function GetTPA81TemperatureFlags
' 
' Returns a byte of 1-bit flags which indicate when the temperatures are
' not within a delta range of the ambient temperature
'
' Parm1 Delta temperature
'**************************************************************
Public Function GetTPA81TemperatureFlags(ByVal Delta as Byte) as Byte
	Dim t(1 to 8) as Byte, i as byte, ambient as Byte
	
	' setup an array of 8 bits to alias the returned byte
	Dim b(1 to 8) as Bit Alias GetTPA81TemperatureFlags
	
	' retrieve the temperatures and ambient temperature
	Call GetTPA81Temperatures(t)
	ambient = GetTPA81AmbientTemperature()
	
	GetTPA81TemperatureFlags = 0
	For i=1 to 8
		' set the bit flag is the temperature difference exceeds the delta
		If abs(CInt(t(i)) - CInt(ambient)) > CInt(Delta) Then
			b(i) = 1
		End If	
	Next
End Function


' *************************************************************
' Sub SetTPA81ServoPosition
' 
' Parm1 is the servo position as a byte value 0 to 31
'**************************************************************
Public Sub SetTPA81ServoPosition(ByVal pos as Byte)
	If pos < 0 or pos > 31 Then
		Debug.Print "Invalid position parameter for SetTPA81ServoPosition"
	Else		
		If sendCmd(address, REG_CMD, pos) < 0 Then
			Debug.Print "SetTPA81ServoPosition i2cmd returned error"
		End If
	End If
End Sub


' *************************************************************
' Sub SetTPA81ServoRange
'
' Parm1 is one end of the servo range in microseconds
' Parm2 is the other end of the servo range in microseconds
'**************************************************************
Public Sub SetTPA81ServoRange(ByVal range1 as Integer, ByVal range2 as Integer)
	Dim range as Byte
	
	range = CByte(min(abs(range1-RANGE_MIDPOINT), abs(range2-RANGE_MIDPOINT)) * 4 \ 31)
	If sendCmd(address, REG_RANGE, range) < 0 Then
		Debug.Print "SetTPA81ServoRange i2cmd returned error"
	End If
End Sub


' *************************************************************
' Function GetTPA81Version
' 
' Returns the software version of the TPA81 in a byte
' Parm1 is the I2C address of the TPA81
'**************************************************************
Public Function GetTPA81Version(ByVal addr as Byte) As Byte
	Dim rc as Integer
	Dim reg as Byte
	reg = REG_REV
	rc = I2CCmd(channel, addr, CByte(SizeOf(reg)), reg, 1, GetTPA81Version)
	If rc < 1 Then
		Debug.Print "GetTPA81Version i2cmd returned ";CStr(rc)
		GetTPA81Version = 0
	End If	
End Function


' *************************************************************
' Function SetTPA81Address
' 
' Sets a new address for the TPA81 and returns the current address
' Parm1 is the current I2C address of the TPA81
' Parm2 is the new I2C address
'**************************************************************
Public Function SetTPA81Address(ByVal addr as Byte, ByVal newAddr as Byte) as Byte
	SetTPA81Address = addr
	If sendCmd(addr, REG_CMD, &HA0) < 0 Then
		Debug.Print "SetTPA81addr i2cmd A0 returned error"
		Exit Function
	End If
	Call Sleep(0.05)
	If sendCmd(addr, REG_CMD, &HAA) < 0 Then
		Debug.Print "SetTPA81addr i2cmd AA returned error"
		Exit Function
	End If	
	Call Sleep(0.05)
	If sendCmd(addr, REG_CMD, &HA5) < 0 Then
		Debug.Print "SetTPA81addr i2cmd A5 returned error"
		Exit Function
	End If	
	Call Sleep(0.05)
	If sendCmd(addr, REG_CMD, newAddr) < 0 Then
		Debug.Print "SetTPA81addr i2cmd set address returned error"
		Exit Function
	End If	
	SetTPA81Address = newAddr
End Function


' *************************************************************
' Define Private Constant Values for TPA81
' *************************************************************

' Define TPA81 Registers
Private Const REG_CMD  As Byte = &H00		' Command Register (Write)
Private Const REG_REV  As Byte = &H00		' Revision Register (Read)
Private Const REG_RANGE As Byte = &H01  	' Servo Range Register (Write)
Private Const REG_AMBIENT As Byte = &H01    ' Ambient Temperature (Read)
Private Const REG_TEMP As Byte = &H02		' Pixel 1 Temperature (Read)
Private Const RANGE_MIDPOINT as Integer = 1500 ' Servo midpoint is 1500 us

' I2C channel constants
Private Const channel as Byte = 0
Private Const sdaPin as Byte = C.1
Private Const sclPin as Byte = C.0

' Private data for TPA81
Private address as Byte
Private temperatureUnits as Byte

' *************************************************************
' Private Function sendCmd
' 
' This routine sends a single byte command to a TPA81 register
' Parm1 is the I2C address of the TPA81
' Parm2 is the TPA81 register number
' Parm3 is the value to write into the register
'**************************************************************
Private Function sendCmd(ByVal addr as Byte, ByVal reg as Byte, ByVal command as Byte) as Integer
	Dim cmd(1 to 2) as Byte	
	cmd(1) = reg
	cmd(2) = command
	SendCmd = I2CCmd(channel, addr, CByte(SizeOf(cmd)), cmd, 0, 0)
End Function